package com.qmth.themis.business.util;

import com.qmth.themis.business.constant.SpringContextHolder;
import com.qmth.themis.business.constant.SystemConstant;
import com.qmth.themis.common.enums.Platform;
import com.qmth.themis.common.enums.Source;
import org.springframework.data.redis.core.RedisTemplate;

import java.util.Objects;
import java.util.concurrent.TimeUnit;

/**
 * @Description: redis util
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/4/15
 */
public class RedisUtil {

    /**
     * 获取用户信息
     *
     * @param userId
     * @return
     */
    public static Object getUser(Long userId) {
        RedisTemplate redisTemplate = SpringContextHolder.getBean(RedisTemplate.class);
        return redisTemplate.opsForValue().get(SystemConstant.USER + userId);
    }

    /**
     * 获取用户会话信息
     *
     * @param sessionId
     * @return
     */
    public static Object getUserSession(String sessionId) {
        RedisTemplate redisTemplate = SpringContextHolder.getBean(RedisTemplate.class);
        return redisTemplate.opsForValue().get(SystemConstant.SESSION + sessionId);
    }

    /**
     * 删除用户缓存
     *
     * @param userId
     */
    public static void deleteUser(Long userId) {
        RedisTemplate redisTemplate = SpringContextHolder.getBean(RedisTemplate.class);
        redisTemplate.delete(SystemConstant.USER + userId);
    }

    /**
     * 删除用户会话缓存
     *
     * @param sessionId
     */
    public static void deleteUserSession(String sessionId) {
        RedisTemplate redisTemplate = SpringContextHolder.getBean(RedisTemplate.class);
        redisTemplate.delete(SystemConstant.SESSION + sessionId);
    }

    /**
     * 设置用户信息
     *
     * @param userId
     * @param o
     */
    public static void setUser(Long userId, Object o) {
        RedisTemplate redisTemplate = SpringContextHolder.getBean(RedisTemplate.class);
        redisTemplate.opsForValue().set(SystemConstant.USER + userId, o, SystemConstant.JWT_WEB_EXPIRE_TIME, TimeUnit.SECONDS);
    }

    /**
     * 设置用户信息
     *
     * @param userId
     * @param platform
     * @param o
     */
    public static void setUser(Long userId, Platform platform, Object o) {
        RedisTemplate redisTemplate = SpringContextHolder.getBean(RedisTemplate.class);
        if (Objects.equals(platform.getSource(), Source.Phone.name())) {
            redisTemplate.opsForValue().set(SystemConstant.USER + userId, o, SystemConstant.JWT_PAD_EXPIRE_TIME, TimeUnit.SECONDS);
        } else {
            redisTemplate.opsForValue().set(SystemConstant.USER + userId, o, SystemConstant.expireTime(platform), TimeUnit.SECONDS);
        }
    }

    /**
     * 刷新用户缓存
     *
     * @param userId
     */
    public static void refreshUser(Long userId) {
        RedisTemplate redisTemplate = SpringContextHolder.getBean(RedisTemplate.class);
        redisTemplate.expire(SystemConstant.USER + userId, SystemConstant.JWT_WEB_EXPIRE_TIME, TimeUnit.SECONDS);
    }

    /**
     * 刷新用户缓存
     *
     * @param userId
     * @param platform
     */
    public static void refreshUser(Long userId, Platform platform) {
        RedisTemplate redisTemplate = SpringContextHolder.getBean(RedisTemplate.class);
        if (Objects.equals(platform.getSource(), Source.Phone.name())) {
            redisTemplate.expire(SystemConstant.USER + userId, SystemConstant.JWT_PAD_EXPIRE_TIME, TimeUnit.SECONDS);
        } else {
            redisTemplate.expire(SystemConstant.USER + userId, SystemConstant.expireTime(platform), TimeUnit.SECONDS);
        }
    }

    /**
     * 获取用户缓存过期时间
     *
     * @param userId
     * @return
     */
    public static Long getUserExpire(Long userId) {
        RedisTemplate redisTemplate = SpringContextHolder.getBean(RedisTemplate.class);
        return redisTemplate.getExpire(SystemConstant.USER + userId, TimeUnit.SECONDS);
    }

    /**
     * 设置用户session信息
     *
     * @param sessionId
     * @param platform
     * @param o
     */
    public static void setUserSession(String sessionId, Platform platform, Object o) {
        RedisTemplate redisTemplate = SpringContextHolder.getBean(RedisTemplate.class);
        if (Objects.equals(platform.getSource(), Source.Phone.name())) {
            redisTemplate.opsForValue().set(SystemConstant.SESSION + sessionId, o, SystemConstant.JWT_PAD_EXPIRE_TIME, TimeUnit.SECONDS);
        } else {
            redisTemplate.opsForValue().set(SystemConstant.SESSION + sessionId, o, SystemConstant.expireTime(platform), TimeUnit.SECONDS);
        }
    }

    /**
     * 刷新用户session缓存
     *
     * @param sessionId
     * @param platform
     */
    public static void refreshUserSession(String sessionId, Platform platform) {
        RedisTemplate redisTemplate = SpringContextHolder.getBean(RedisTemplate.class);
        if (Objects.equals(platform.getSource(), Source.Phone.name())) {
            redisTemplate.expire(SystemConstant.SESSION + sessionId, SystemConstant.JWT_PAD_EXPIRE_TIME, TimeUnit.SECONDS);
        } else {
            redisTemplate.expire(SystemConstant.SESSION + sessionId, SystemConstant.expireTime(platform), TimeUnit.SECONDS);
        }
    }

    /**
     * 获取用户session缓存过期时间
     *
     * @param sessionId
     * @return
     */
    public static Long getUserSessionExpire(String sessionId) {
        RedisTemplate redisTemplate = SpringContextHolder.getBean(RedisTemplate.class);
        return redisTemplate.getExpire(SystemConstant.SESSION + sessionId, TimeUnit.SECONDS);
    }
}